#include <ilcplex/cplex.h>
#include <stdlib.h>

static void free_and_null(char **ptr);

int main(int argc, char **argv)
{
	int i, j;
	int cur_numrows, cur_numcols;
	int solstat;
	int status = 0;

	double objval;

	double *x     = NULL;
	double *pi    = NULL;
	double *slack = NULL;
	double *dj    = NULL;

	CPXENVptr env = NULL;
	CPXLPptr  lp = NULL;

	/* Inicializa o ambiente do CPLEX */

	env = CPXopenCPLEX(&status);

	/* Se ocorrer um erro, o valor do status indica o motivo da falha.
	   Uma chamada para CPXgeterrorstring produzir o texto da mensagem de erro.
	   Observe que o CPXopenCPLEX no produz sada, portanto, a nica
	   maneira de ver a causa do erro  usar o CPXgeterrorstring.
	   Para outras rotinas do CPLEX, os erros sero vistos se o indicador
	   CPXPARAM_ScreenOutput estiver definido como CPX_ON. */

	if (env == NULL)
	{
		char errmsg[CPXMESSAGEBUFSIZE];

		fprintf(stderr, "No foi possvel abrir o ambiente CPLEX.\n");
		CPXgeterrorstring(env, status, errmsg);
		fprintf(stderr, "%s", errmsg);

		goto TERMINATE;
	}

	/* Ativa a sada na tela */

	status = CPXsetintparam(env, CPXPARAM_ScreenOutput, CPX_ON);

	if (status)
	{
		fprintf(stderr, "Falha ao ativar o indicador na tela, erro %d.\n", status);
		goto TERMINATE;
	}

	/* Ativa a verificao de dados */

	status = CPXsetintparam(env, CPXPARAM_Read_DataCheck, CPX_DATACHECK_WARN);

	if (status)
	{
		fprintf(stderr, "Falha ao ativar a verificao de dados, erro %d.\n", status);
		goto TERMINATE;
	}

	/* Cria o problema, usando o nome do arquivo como nome do problema */

	lp = CPXcreateprob(env, &status, argv[1]);

	/* Um ponteiro NULL retornado pode significar que no h memria suficiente
	   disponvel ou houve algum outro problema. No caso de falha,
	   uma mensagem de erro ser gravada no canal de erro de dentro do CPLEX.
	   Neste exemplo, a configurao do parmetro CPXPARAM_ScreenOutput faz
	   com que a mensagem de erro aparea no stdout. */

	if (lp == NULL)
	{
		fprintf(stderr, "Falha ao criar o LP.\n");
		goto TERMINATE;
	}

	/* L o arquivo e copia os dados no LP criado */

	status = CPXreadcopyprob(env, lp, argv[1], NULL);

	if (status)
	{
		fprintf(stderr, "Falha ao ler e copiar os dados do problema.\n");
		goto TERMINATE;
	}

	/* Otimiza o problema e obtm a soluo. */

	status = CPXlpopt(env, lp);

	if (status)
	{
		fprintf(stderr, "Falha ao otimizar o LP.\n");
		goto TERMINATE;
	}

	/* O tamanho do problema deve ser obtido perguntando ao CPLEX qual  o
	   tamanho real, em vez de usar tamanhos de quando o problema foi construdo.
	   cur_numrows e cur_numcols armazenam o nmero atual de linhas e colunas,
	   respectivamente. */

	cur_numrows = CPXgetnumrows(env, lp);
	cur_numcols = CPXgetnumcols(env, lp);

	x     = (double *)malloc(cur_numcols * sizeof(double));
	slack = (double *)malloc(cur_numrows * sizeof(double));
	dj    = (double *)malloc(cur_numcols * sizeof(double));
	pi    = (double *)malloc(cur_numrows * sizeof(double));

	if (x == NULL || slack == NULL || dj == NULL || pi == NULL)
	{
		status = CPXERR_NO_MEMORY;
		fprintf(stderr, "No foi possvel alocar memria para a soluo.\n");
		goto TERMINATE;
	}

	status = CPXsolution(env, lp, &solstat, &objval, x, pi, slack, dj);

	if (status)
	{
		fprintf(stderr, "Falha ao obter a soluo.\n");
		goto TERMINATE;
	}

	/* Escreve a sada na tela. */

	printf("\n");
	printf("Solution status = %d\n", solstat);
	printf("Solution value  = %f\n\n", objval);

	for (i = 0; i < cur_numrows; i++)
	{
		printf("Row %d:  Slack = %lf  Pi = %lf\n", i, slack[i], pi[i]);
	}

	for (j = 0; j < cur_numcols; j++)
	{
		printf("Column %d:  Value = %lf  Reduced cost = %lf\n", j, x[j], dj[j]);
	}

TERMINATE:

	/* Libera a memria alocada para a soluo */

	free_and_null((char **)&x);
	free_and_null((char **)&slack);
	free_and_null((char **)&dj);
	free_and_null((char **)&pi);

	/* Libera o problema alocado pelo CPXcreateprob, se necessrio */

	if (lp != NULL)
	{
		status = CPXfreeprob(env, &lp);

		if (status)
		{
			fprintf(stderr, "CPXfreeprob falhou, cdigo de erro %d.\n", status);
		}
	}

	/* Libera o ambiente do CPLEX, se necessrio */

	if (env != NULL)
	{
		status = CPXcloseCPLEX(&env);

		/* Observe que o CPXcloseCPLEX no produz sada; portanto,
		   a nica maneira de ver a causa do erro  usar o CPXgeterrorstring.
		   Para outras rotinas do CPLEX, os erros sero vistos se o indicador
		   CPXPARAM_ScreenOutput estiver definido como CPX_ON. */

		if (status)
		{
			char errmsg[CPXMESSAGEBUFSIZE];

			fprintf(stderr, "No foi possvel fechar o ambiente CPLEX.\n");
			CPXgeterrorstring(env, status, errmsg);
			fprintf(stderr, "%s", errmsg);
		}
	}

	return (status);
}

static void free_and_null(char** ptr)
{
	if (*ptr != NULL)
	{
		free(*ptr);
		*ptr = NULL;
	}
}